import pyglet

from typing import Tuple, List


class EffectSprite(pyglet.sprite.Sprite):
    _done = False

    def on_animation_end(self):
        self.delete()
        self._done = True

    @property
    def done(self):
        return self._done


class HPBar:
    def __init__(self, batch, obj_left_x, obj_bottom_y, obj_width, offset_width, offset_height, thickness, initial_hp):
        self.obj_width = obj_width
        self.offset_width = offset_width
        self.hp_sprite = pyglet.shapes.Rectangle(
            obj_left_x + offset_width,
            obj_bottom_y + offset_height, 
            self._hp_bar_width(initial_hp, initial_hp),
            thickness, 
            color=self._hp_bar_color(initial_hp, initial_hp), 
            batch=batch
        )

    def update(self, dt: float, obj_left_x, initial_hp, hp):
        self.hp_sprite.x = obj_left_x + self.offset_width
        self.hp_sprite.width = self._hp_bar_width(hp, initial_hp)
        self.hp_sprite.color = self._hp_bar_color(hp, initial_hp)


    def _hp_bar_width(self, hp, initial_hp) -> float:
        return (self.obj_width - self.offset_width) * hp / initial_hp

    def _hp_bar_color(self, hp, initial_hp) -> Tuple[int, int, int]:
        hp_ratio = hp / initial_hp
        return (
            int(200 - 100*hp_ratio), 
            int(100 + 100*hp_ratio), 
            100
        )


class HealthEffect(EffectSprite):
    pass


class HealthEffectFactory:
    def __init__(self, health_effect_image, batch, effects: List):
        self.health_effect_image = health_effect_image
        self.batch = batch
        self.effects = effects

    def create(self, x, y):
        effect_sprite = EffectSprite(
                self.health_effect_image,
                x,
                y,
                batch=self.batch
            )
        effect_sprite.scale = 5.0
        self.effects.append(effect_sprite)
        return effect_sprite


class ShieldEffectFactory:
    def __init__(self, shield_effect_image, batch, effects: List):
        self.shield_effect_image = shield_effect_image
        self.batch = batch
        self.effects = effects

    def create(self, x, y):
        effect_sprite = EffectSprite(
                self.shield_effect_image,
                x,
                y,
                batch=self.batch
            )
        effect_sprite.scale = 5.0
        self.effects.append(effect_sprite)
        return effect_sprite
        